@@ -1,4 +1,4 @@
-#' @include utils.R
+#' @include utils.R stack.R
 NULL
 
 #' Web Application Framework for R
@@ -15,7 +15,7 @@ NULL
 #' @name shiny-package
 #' @aliases shiny
 #' @docType package
-#' @import htmltools httpuv caTools xtable digest methods
+#' @import htmltools httpuv xtable digest R6 mime
 #' @importFrom RJSONIO fromJSON
 NULL
 
@@ -175,15 +175,17 @@ NULL
 
 
 #' @include utils.R
-ShinySession <- setRefClass(
+ShinySession <- R6Class(
   'ShinySession',
-  fields = list(
+  portable = FALSE,
+  class = FALSE,
+  public = list(
     .websocket = 'ANY',
     .invalidatedOutputValues = 'Map',
     .invalidatedOutputErrors = 'Map',
-    .inputMessageQueue = 'list',    # A list of inputMessages to send when flushed
-    .outputs = 'list',       # Keeps track of all the output observer objects
-    .outputOptions = 'list', # Options for each of the output observer objects
+    .inputMessageQueue = list(),    # A list of inputMessages to send when flushed
+    .outputs = list(),       # Keeps track of all the output observer objects
+    .outputOptions = list(), # Options for each of the output observer objects
     .progressKeys = 'character',
     .showcase   = 'ANY',
     .fileUploadContext = 'FileUploadContext',
@@ -192,31 +194,40 @@ ShinySession <- setRefClass(
     .closedCallbacks = 'Callbacks',
     .flushCallbacks = 'Callbacks',
     .flushedCallbacks = 'Callbacks',
+    progressStack = 'Stack', # Stack of progress objects
     input       = 'reactivevalues', # Externally-usable S3 wrapper object for .input
     output      = 'ANY',    # Externally-usable S3 wrapper object for .outputs
     clientData  = 'reactivevalues', # Externally-usable S3 wrapper object for .clientData
     token = 'character',  # Used to identify this instance in URLs
     files = 'Map',        # For keeping track of files sent to client
     downloads = 'Map',
-    closed = 'logical',
+    closed = logical(0),
     session = 'environment',      # Object for the server app to access session stuff
-    singletons = 'character'  # Tracks singleton HTML fragments sent to the page
-  ),
-  methods = list(
+    singletons = character(0),  # Tracks singleton HTML fragments sent to the page
+
     initialize = function(websocket) {
       .websocket <<- websocket
       closed <<- FALSE
       # TODO: Put file upload context in user/app-specific dir if possible
 
+      .invalidatedOutputValues <<- Map$new()
+      .invalidatedOutputErrors <<- Map$new()
+      .fileUploadContext <<- FileUploadContext$new()
+      .closedCallbacks <<- Callbacks$new()
+      .flushCallbacks <<- Callbacks$new()
+      .flushedCallbacks <<- Callbacks$new()
       .input      <<- ReactiveValues$new()
       .clientData <<- ReactiveValues$new()
+      progressStack <<- Stack$new()
+      files <<- Map$new()
+      downloads <<- Map$new()
 
       input      <<- .createReactiveValues(.input,      readonly=TRUE)
       .setLabel(input, 'input')
       clientData <<- .createReactiveValues(.clientData, readonly=TRUE)
       .setLabel(clientData, 'clientData')
 
-      output     <<- .createOutputWriter(.self)
+      output     <<- .createOutputWriter(self)
 
       token <<- createUniqueId(16)
       .outputs <<- list()
@@ -224,18 +235,20 @@ ShinySession <- setRefClass(
 
       session <<- new.env(parent=emptyenv())
       session$clientData        <<- clientData
-      session$sendCustomMessage <<- .self$.sendCustomMessage
-      session$sendInputMessage  <<- .self$.sendInputMessage
-      session$onSessionEnded    <<- .self$onSessionEnded
-      session$onEnded           <<- .self$onEnded
-      session$onFlush           <<- .self$onFlush
-      session$onFlushed         <<- .self$onFlushed
-      session$isClosed          <<- .self$isClosed
-      session$input             <<- .self$input
-      session$output            <<- .self$output
-      session$reactlog          <<- .self$reactlog
-      session$registerDataObj   <<- .self$registerDataObj
-      session$.impl             <<- .self
+      session$sendCustomMessage <<- self$.sendCustomMessage
+      session$sendInputMessage  <<- self$.sendInputMessage
+      session$onSessionEnded    <<- self$onSessionEnded
+      session$onEnded           <<- self$onEnded
+      session$onFlush           <<- self$onFlush
+      session$onFlushed         <<- self$onFlushed
+      session$isClosed          <<- self$isClosed
+      session$input             <<- self$input
+      session$output            <<- self$output
+      session$reactlog          <<- self$reactlog
+      session$registerDataObj   <<- self$registerDataObj
+      session$progressStack     <<- self$progressStack
+      session$sendProgress      <<- self$sendProgress
+      session$.impl             <<- self
 
       if (!is.null(websocket$request$HTTP_SHINY_SERVER_CREDENTIALS)) {
         try({
@@ -312,7 +325,7 @@ ShinySession <- setRefClass(
         if (length(formals(func)) != 0) {
           orig <- func
           func <- function() {
-            orig(name=name, shinysession=.self)
+            orig(name=name, shinysession=self)
           }
         }
 
@@ -417,15 +430,19 @@ ShinySession <- setRefClass(
 
       .progressKeys <<- c(.progressKeys, id)
 
-      json <- toJSON(list(progress=list(id)))
-
+      sendProgress('binding', list(id = id))
+    },
+    sendProgress = function(type, message) {
+      json <- toJSON(list(
+        progress = list(type = type, message = message)
+      ))
       .write(json)
     },
     dispatch = function(msg) {
       method <- paste('@', msg$method, sep='')
       # we must use $ instead of [[ here at the moment; see
       # https://github.com/rstudio/shiny/issues/274
-      func <- try(do.call(`$`, list(.self, method)), silent=TRUE)
+      func <- try(do.call(`$`, list(self, method)), silent=TRUE)
       if (inherits(func, 'try-error')) {
         .sendErrorResponse(msg, paste('Unknown method', msg$method))
       }
@@ -504,6 +521,9 @@ ShinySession <- setRefClass(
     },
 
     # Public RPC methods
+    `@uploadieFinish` = function() {
+      # Do nothing; just want the side effect of flushReact, output flush, etc.
+    },
     `@uploadInit` = function(fileInfos) {
       maxSize <- getOption('shiny.maxRequestSize', 5 * 1024 * 1024)
       fileInfos <- lapply(fileInfos, function(fi) {
@@ -566,6 +586,13 @@ ShinySession <- setRefClass(
         }
       }
 
+      if (matches[2] == 'uploadie' && identical(req$REQUEST_METHOD, "POST")) {
+        id <- URLdecode(matches[3])
+        res <- mime::parse_multipart(req)
+        .input$set(id, res[[id]])
+        return(httpResponse(200, 'text/plain', 'OK'))
+      }
+
       if (matches[2] == 'download') {
 
         # A bunch of ugliness here. Filenames can be dynamically generated by
@@ -671,7 +698,7 @@ ShinySession <- setRefClass(
       fileData <- readBin(file, 'raw', n=bytes)
 
       if (isTRUE(.clientData$.values$allowDataUriScheme)) {
-        b64 <- base64encode(fileData)
+        b64 <- rawToBase64(fileData)
         return(paste('data:', contentType, ';base64,', b64, sep=''))
       } else {
         return(saveFileUrl(name, fileData, contentType))
